@@ -1,5 +1,10 @@
 
-# $Id: Ebay.pm,v 2.262 2013/08/21 01:15:16 Martin Exp $
+# $Id: Ebay.pm,v 2.271 2015-09-13 14:28:54 Martin Exp $
+
+package WWW::Search::Ebay;
+
+use strict;
+use warnings;
 
 =head1 NAME
 
@@ -53,9 +58,6 @@ field is a string containing the high bid or starting bid as a
 human-readable monetary value in seller-native units, e.g. "$14.95" or
 "GBP 6.00".
 
-In the resulting L<WWW::Search::Result> objects, the category() field
-contains the Ebay category number.
-
 In the resulting L<WWW::Search::Result> objects, the sold() field will
 be non-zero if the item has already sold.  (Only if you're using
 WWW::Search::Ebay::Completed)
@@ -67,7 +69,7 @@ the count of items matching your query in each category and
 subcategory.  (Special thanks to Nick Lokkju for this code!)  For
 example:
 
-  $oSearch->{category} = [
+  $oSearch->{categories} = [
           {
             'ID' => '1',
             'Count' => 19,
@@ -116,11 +118,6 @@ Contributed by Brian Wilson:
 
 =cut
 
-package WWW::Search::Ebay;
-
-use strict;
-use warnings;
-
 use base 'WWW::Search';
 
 use constant DEBUG_DATES => 0;
@@ -130,7 +127,7 @@ use Carp ();
 use CGI;
 use Data::Dumper;  # for debugging only
 use Date::Manip;
-&Date_Init('TZ=US/Pacific') unless (defined($ENV{TZ}) && ($ENV{TZ} ne ''));
+# Date_Init("setdate=now,America/Los_Angeles");
 use HTML::TreeBuilder;
 use LWP::Simple;
 use WWW::Search qw( generic_option strip_tags );
@@ -139,7 +136,7 @@ use WWW::SearchResult 2.072;
 use WWW::Search::Result;
 
 our
-$VERSION = do { my @r = (q$Revision: 2.262 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
+$VERSION = 2.272;
 our $MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
 my $cgi = new CGI;
 
@@ -368,6 +365,26 @@ sub _create_description
   return $sDesc;
   } # _create_description
 
+sub _parse_category
+  {
+  my $self = shift;
+  my $oTD = shift;
+  return -1 if ! ref $oTD;
+  my $oA = $oTD->look_down(_tag => 'a');
+  return -1 if ! ref $oA;
+  if (DEBUG_COLUMNS || (1 < $self->{_debug}))
+    {
+    my $s = $oA->as_HTML;
+    print STDERR " DDD   TDcategory's A ===$s===\n";
+    } # if
+  my $sURL = $oA->attr('href') || q{};
+  if ($sURL =~ m/sibeleafcat=(\d+)/)
+    {
+    return $1;
+    } # if
+  return -1;
+  } # _parse_category
+
 sub _parse_price
   {
   my $self = shift;
@@ -397,8 +414,13 @@ sub _parse_price
     return 0;
     # There is a separate backend for searching Auction items!
     } # if
-  if ($oTDprice->look_down(_tag => 'span',
-                          class => 'ebSold'))
+  if (
+      $oTDprice->look_down(_tag => 'span',
+                          class => 'ebSold')
+      ||
+      $oTDprice->look_down(_tag => 'span',
+                          class => 'bold bidsold')
+     )
     {
     # This item sold, even if it had no bids (i.e. Buy-It-Now)
     $hit->sold(1);
@@ -412,6 +434,8 @@ sub _parse_price
   my $iPrice = $oTDprice->as_text;
   print STDERR " DDD   raw iPrice ===$iPrice===\n" if  (DEBUG_COLUMNS || (1 < $self->{_debug}));
   $iPrice =~ s!&pound;!GBP!;
+  $iPrice =~ s!\s*Trending.+!!;
+  $iPrice =~ s!\s*Was.+!!;
   # Convert nbsp to regular space:
   $iPrice =~ s!\240!\040!g;
   # I don't know why there are sometimes weird characters in there:
@@ -503,6 +527,10 @@ sub _parse_shipping
   $iPrice =~ s!&Acirc;!!g;
   $iPrice =~ s!!!g;
   print STDERR " DDD   raw shipping ===$iPrice===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+  if ($iPrice =~ m/FREE/i)
+    {
+    $iPrice = 0.00;
+    } # if
   return 0 if ($iPrice !~ m/\d/);
   $iPrice =~ s!&pound;!GBP!;
   $hit->shipping($iPrice);
@@ -539,6 +567,7 @@ sub _parse_enddate
     {
     $sDate = $1;
     $sDate = $self->_format_date(ParseDate(q{epoch }. int($sDate/1000)));
+    print STDERR " DDD   sDate =$sDate=\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
     $hit->end_date($sDate);
     # For backward-compatibility:
     $hit->change_date($sDate);
@@ -546,7 +575,8 @@ sub _parse_enddate
     }
   if (ref($oTDdate))
     {
-    if ($oTDdate->attr('class') !~ m/\b(col3|ebcTim|ti?me)\b/)
+    my $sClass = $oTDdate->attr('class') || q{};
+    if ($sClass !~ m/\b(col3|ebcTim|ti?me)\b/)
       {
       # If we see this, we probably were searching for Buy-It-Now items
       # but we ran off the bottom of the item list and ran into the list
@@ -603,7 +633,7 @@ sub result_as_HTML
   my $self = shift;
   my $oSR = shift or return '';
   my $sDateFormat = shift || q'%Y-%m-%d %H:%M:%S';
-  my $dateEnd = ParseDate($oSR->end_date);
+  my $dateEnd = ParseDate($oSR->end_date) || q{};
   my $iItemNum = $oSR->item_number;
   my $sSold = $oSR->sold
   ? $cgi->font({color=>'green'}, 'sold') .q{; }
@@ -662,10 +692,15 @@ sub _get_result_count_elements
   {
   my $self = shift;
   my $tree = shift;
-  my @ao = $tree->look_down(
-                            '_tag' => 'div',
-                            class => 'fpcc'
-                           );
+  my @ao;
+  push @ao, $tree->look_down( # as of 2015-06
+                             '_tag' => 'span',
+                             class => 'listingscnt'
+                            );
+  push @ao, $tree->look_down(
+                             '_tag' => 'div',
+                             class => 'fpcc'
+                            );
   push @ao, $tree->look_down(
                              '_tag' => 'div',
                              class => 'fpc'
@@ -725,6 +760,9 @@ sub _get_itemtitle_tds
     push @ao, $oDiv->look_down(_tag => 'td',
                                class => 'dtl dtlsp',
                               );
+    push @ao, $oDiv->look_down(_tag => 'h3',
+                               class => 'lvtitle',
+                              );
     } # if
   return @ao;
   } # _get_itemtitle_tds
@@ -756,7 +794,7 @@ sub _parse_tree
     $self->approximate_result_count(1);
     return 1;
     } # if
-  my $iHits = 0;
+
   # First, see if: there were zero results and eBay automatically did
   # a spell-check and searched for other words (or searched for a
   # subset of query terms):
@@ -781,6 +819,7 @@ sub _parse_tree
       return 0;
       } # if
     } # if
+
   # See if our query was completely replaced by a similar-spelling query:
   my $oLI = $tree->look_down(_tag => 'li',
                              class => 'ebInf',
@@ -793,10 +832,30 @@ sub _parse_tree
       return 0;
       } # if
     } # if
-  # The hit count is in a FONT tag:
-  my @aoFONT = $self->_get_result_count_elements($tree);
+
+  # See if our category-only query was replaced by a global query:
+  my $oP = $tree->look_down(_tag => 'p',
+                             class => 'sm-md',
+                            );
+  if (ref $oP)
+    {
+    my $s = $oP->as_text;
+    if (($s =~ m/0 results found in the/) && ($s =~ m/so we searched in all categories/))
+      {
+      return 0;
+      } # if
+    } # if
+
+  my $iHits = 0;
+
+  # The hit count is in one of these tags:
+  my @aoResultCountTagset = $self->_get_result_count_elements($tree);
+  if (scalar(@aoResultCountTagset) < 1)
+    {
+    warn " EEE no result_count_elements matched the HTML\n";
+    } # if
  FONT:
-  foreach my $oFONT (@aoFONT)
+  foreach my $oFONT (@aoResultCountTagset)
     {
     my $qr = $self->_result_count_pattern;
     print STDERR (" DDD   result_count try ==",
@@ -807,11 +866,16 @@ sub _parse_tree
       print STDERR " DDD     matched ($sCount)\n" if (1 < $self->{_debug});
       # Make sure it's an integer:
       $sCount =~ s!,!!g;
-      $self->approximate_result_count($sCount);
+      $self->approximate_result_count(0 + $sCount);
       last FONT;
       } # if
     } # foreach
 
+  if ($self->approximate_result_count() < 1)
+    {
+    return $iHits;
+    } # if
+
   # Recursively parse the stats telling how many items were found in
   # each category:
   my $oUL = $tree->look_down(_tag => 'ul',
@@ -829,32 +893,6 @@ sub _parse_tree
     $oDiv->detach;
     $oDiv->delete;
     } # if
-  # By default, use the hard-coded order of columns:
-  my @asColumns = $self->_columns;
-  if (ref($self) !~ m!::Completed!)
-    {
-    # See if we can glean the actual order of columns from the page itself:
-    my @aoCOL = $tree->look_down(_tag => 'col');
-    my @asId;
-    foreach my $oCOL (@aoCOL)
-      {
-      # Sanity check:
-      next unless ref($oCOL);
-      my $sId = $oCOL->attr('id') || '';
-      # Sanity check:
-      next unless ($sId ne '');
-      $sId =~ s!\Aebc!!;
-      # Try not to go past the first table:
-      last if ($sId eq 'bdrRt');
-      push @asId, $sId;
-      } # foreach
-    print STDERR " DDD raw    asId is (@asId)\n" if (1 < $self->{_debug});
-    1 while (@asId && (shift(@asId) ne 'title'));
-    local $" = ',';
-    print STDERR " DDD cooked asId is (@asId)\n" if (1 < $self->{_debug});
-    @asColumns = @asId if @asId;
-    } # if
-  print STDERR " DDD   asColumns is (@asColumns)\n" if (1 < $self->{_debug});
   # The list of matching items is in a table.  The first column of the
   # table is nothing but icons; the second column is the good stuff.
   my @aoTD = $self->_get_itemtitle_tds($tree);
@@ -883,9 +921,9 @@ sub _parse_tree
     # next TD unless ($oURI =~ m!ViewItem!);
     next TD if ($oURI !~ m!$qrItemNum!);
     my $iItemNum = $1;
+    print STDERR " DDD   iItemNum ===$iItemNum===\n" if (1 < $self->{_debug});
     my $iCategory = -1;
     $iCategory = $1 if ($oURI =~ m!QQcategoryZ(\d+)QQ!);
-    print STDERR " DDD   iItemNum ===$iItemNum===\n" if (1 < $self->{_debug});
     if ($oURI->as_string =~ m!QQitemZ(\d+)QQ!)
       {
       # Convert new eBay links to old reliable ones:
@@ -897,98 +935,66 @@ sub _parse_tree
     my $hit = new WWW::Search::Result;
     $hit->add_url($self->_cleanup_url($sURL));
     $hit->title($sTitle);
-    $hit->category($iCategory);
     $hit->item_number($iItemNum);
     # This is just to prevent undef warnings later on:
     $hit->bid_count(0);
-    # The rest of the info about this item is in sister TD elements to
-    # the right:
-    my @aoSibs = $oTDtitle->right;
-    # But in the Completed auctions list, the rest of the info is in
-    # the next row of the table:
-    if (0 && ref($self) =~ m!::Completed!)
-      {
-      @aoSibs = ();
-      my $oTRparent = $oTDtitle->look_up(_tag => 'tr');
-      if (ref $oTRparent)
-        {
-        my $sTRparent = $oTRparent->as_HTML;
-        # print STDERR " DDD oTRparent ==$sTRparent==\n";
-        my $oTRaunt = $oTRparent->right;
-        if (ref $oTRaunt)
-          {
-          my $sTRaunt = $oTRaunt->as_HTML;
-          # print STDERR " DDD oTRaunt ==$sTRaunt==\n";
-          @aoSibs = $oTRaunt->look_down(_tag => 'td');
-          # Throw out one empty cell:
-          shift @aoSibs;
-          } # if
-        } # if
-      } # if
-    my $iCol = 0;
-    my $oTDprev;
+    # The rest of the info about this item is in sister <LI> elements
+    # to the right:
+    my @aoSibs = $oTDtitle->parent->look_down(_tag => q{li});
+    # The parent itself is an <LI> tag:
+    shift @aoSibs;
     warn " DDD before loop, there are ", scalar(@aoSibs), " sibling TDs\n" if (1 < $self->{_debug});
  SIBLING_TD:
-    while (my $sColumn = $asColumns[$iCol++])
+    while (my $oTDsib = shift @aoSibs)
       {
-      warn " DDD just inside loop, there are ", scalar(@aoSibs), " sibling TDs\n" if (1 < $self->{_debug});
-      if ($sColumn =~ m/(repeat|stay|same)/i)
-        {
-        # Re-use the previous table cell for the (next) column of data:
-        unshift @aoSibs, $oTDprev;
-        warn " DDD after unshifting, there are ", scalar(@aoSibs), " sibling TDs\n" if (1 < $self->{_debug});
-        next SIBLING_TD;
-        } # if
-      my $oTDsib = shift(@aoSibs);
       next unless ref($oTDsib);
+      my $sColumn = $oTDsib->attr('class') || q{};
       my $s = $oTDsib->as_HTML;
-      print STDERR " DDD   try TD'$sColumn' ===$s===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+      if ($sColumn eq q{})
+        {
+        warn " WWW auction info sibling has no class ==$s==" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+        } # if
+      print STDERR " DDD   looking at TD'$sColumn' ===$s===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
       if ($sColumn =~ m'price')
         {
         next TD unless $self->_parse_price($oTDsib, $hit);
         } # if
-      elsif ($sColumn =~ m'bids')
+      if (($sColumn =~ m'bids') || ($sColumn =~ m'format'))
         {
         # It is not a fatal error if there are no bids (i.e. buy-it-now)
         $self->_parse_bids($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'shipping')
+      if ($sColumn =~ m'shipping')
         {
-        next TD unless $self->_parse_shipping($oTDsib, $hit);
+        next TD if ! $self->_parse_shipping($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'enddate')
+      if ($sColumn =~ m'end')
         {
-        next TD unless $self->_parse_enddate($oTDsib, $hit);
+        next TD if ! $self->_parse_enddate($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'time')
+      if ($sColumn =~ 'time')
         {
-        next TD unless $self->_parse_enddate($oTDsib, $hit);
+        next TD if ! $self->_parse_enddate($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'country')
+      if ($sColumn =~ m'country')
         {
         # This listing is from a country other than the base site
         # we're searching against.  Throw it out:
         next TD;
         }
-      elsif ($sColumn eq 'paypal')
+      if ($sColumn =~ m'extras')
         {
-        # We always ignore the Paypal logo.
-        next SIBLING_TD;
-        }
-      elsif ($sColumn eq 'buyitnowlogo')
-        {
-        # We always ignore the Buy-It-Now logo.
-        next SIBLING_TD;
-        }
-      else
-        {
-        # print STDERR " DDD     do not know how to handle column named $sColumn\n" if (1 < $self->{_debug});
-        next SIBLING_TD;
-        }
-      $oTDprev = $oTDsib;
+        if ($iCategory < 0)
+          {
+          # We haven't found this item's category.  Look for it here:
+          $iCategory = $self->_parse_category($oTDsib);
+          } # if
+        } # if 'extras'
+      # Any other class="" value will cause the <LI> to be ignored.
       } # while
     my $sDesc = $self->_create_description($hit);
     $hit->description($sDesc);
+    $hit->category($iCategory);
     # Clean up / sanity check hit info:
     my ($enddate, $iBids);
     if (
@@ -1019,7 +1025,7 @@ sub _parse_tree
     # AS OF 2008-11 THE NEXT LINK CAN NOT BE FOLLOWED FROM PERL CODE
 
     # Look for a NEXT link:
-    my @aoA = $tree->look_down('_tag', 'a');
+    my @aoA = $tree->look_down('_tag' => 'a');
  TRY_NEXT:
     foreach my $oA (0, reverse @aoA)
       {
@@ -1189,23 +1195,10 @@ Include parentheses so that $1 becomes the number (with commas is OK).
 
 sub _result_count_pattern
   {
-  return qr'([0-9,]+)\s+(active\s+)?(listing|item|matche?|result)s?(\s+found)?\b';
+  return qr'([0-9,]+)\s+(active\s+)?(listing|item|matche?|result)s?(\s+found)?';
   } # _result_count_pattern
 
 
-=item _columns
-
-Specify the order in which data columns appear in the search results table.
-
-=cut
-
-sub _columns
-  {
-  my $self = shift;
-  # This is for basic USA eBay:
-  return qw( enddate price repeat bids );
-  } # _columns
-
 1;
 
 __END__
